#!/usr/bin/env Rscript



masterset_graph_points <- function(
    g_xvar  = NULL,   # X-Axis variable
    g_yvar  = NULL,   # Y-Axis variable

    g_col   = NULL,   # Plot-Colors            (default set by masterset_scope)
    g_size  = NULL,   # Plot-Point size        (default set by dataset size)
    g_cat   = NULL,   # Plot-Category circles  (default set none)
    g_labs  = NULL,   # Plot-Point labels      (default set by masterset_scope)
    
    g_llab  = NULL,   # Legend title/lable     (default set by masterset_scope)
    g_src   = TRUE,   # Show data source       (default set true)
    
    g_xlab  = NULL,   # X-Axis title/label     (default set by masterset_meta)
    g_xunit = NULL,   # X-Axis grid units      (default set by masterset_meta)
    g_xlog  = FALSE,  # X-Axis rescale to logarithmic
    
    g_ylab  = NULL,   # Y-Axis title/label     (default set by masterset_meta)
    g_yunit = NULL,   # Y-Axis grid units      (default set by masterset_meta)
    g_ylog  = FALSE   # Y-Axis rescale to logarithmic

){
  
  
  
  #--------[ Process Data ]--------#
  if( is.null(g_size) ){
    g_size_tmp = 8
    if( length(g_xvar) > 100 ){ g_size_tmp = 6 }
    if( length(g_xvar) > 200 ){ g_size_tmp = 4 }
    if( length(g_xvar) > 300 ){ g_size_tmp = 2 }
  }else{
    g_size_tmp = 1 + (28 * masterset_norm(g_size))
    if( length(g_xvar) > 100 ){ g_size_tmp = 1 + (24 * masterset_norm(g_size)) }
    if( length(g_xvar) > 200 ){ g_size_tmp = 1 + (20 * masterset_norm(g_size)) }
    if( length(g_xvar) > 300 ){ g_size_tmp = 1 + (16 * masterset_norm(g_size)) }
  }
  
  if( is.null(g_col) ){
    if( masterset_scope=='global' ){ g_col = masterset$country.continent }
    if( masterset_scope=='regional/united-states' ){ g_col = masterset$us.county.state }
  }
  
  if( is.null(g_labs) ){
    if( masterset_scope=='global' ){ g_labs = masterset$country.name.short }
    if( masterset_scope=='regional/united-states' ){ g_labs = masterset$us.county.name }
  }
  
  if( is.null(g_llab) ){
    if( is.null(g_col) ){
      if( masterset_scope=='global' ){ g_llab = 'Continent' }
      if( masterset_scope=='regional/united-states' ){ g_llab = 'State' }
    }else{
      g_llab = get_meta(g_col)$name
    }
  }
  
  
  #--------[ Graph Best Fit ]--------#
  masterset_graph <<- masterset_graph + stat_smooth(
    aes(
      x    = g_xvar,
      y    = g_yvar
    ),
    size   = 1,
    method = 'loess',
    geom   = 'line',
    se     = FALSE,
    color  = '#000000',
    alpha  = 0.5,
    na.rm  = TRUE
  )

  
  
  
  #--------[ Graph Points ]--------#
  
  
  masterset_graph <<- masterset_graph + geom_point(
    aes(
      x     = g_xvar,
      y     = g_yvar,
      fill  = g_col,
    ),
    color  = '#000000',
    shape  = 21,
    size   = g_size_tmp,
    alpha  = 0.5
  )
  
  
  
  #--------[ Graph Labels ]--------#
  masterset_graph <<- masterset_graph + geom_text_repel(
    aes(
      x          = g_xvar,
      y          = g_yvar,
    ),
    label        = g_labs,
    max.overlaps = 5,
    size         = 2.5,
    hjust        = 0,
    alpha        = 0.85,
    na.rm        = TRUE,
  )

  

  #--------[ Set X-Axis Scale ]--------#
  if( is.null(g_xlab) ){ g_xlab = get_meta(g_xvar)$name }
  if( is.null(g_xunit) ){ g_xunit = get_meta(g_xvar)$units }
  if( g_xunit!='$' && g_xunit!='%'){ g_xlab = paste(g_xlab,' (',g_xunit,')',sep='') }
  if( g_xlog ){ g_xlab = paste(g_xlab, "    |    Scale: log(10)", sep='')}
  
  if( g_xlog ){
    if( g_xunit=='$' ){
      masterset_graph <<- masterset_graph + scale_x_log10(labels=scales::dollar)
    }else if( g_xunit=='%' ){
      masterset_graph <<- masterset_graph + scale_x_log10(labels=scales::percent)
    }else if( nchar(g_xunit)<=4 ){
      masterset_graph <<- masterset_graph + scale_x_log10(labels=comma_format( suffix=paste(' ',g_xunit) ))
    }else{
      masterset_graph <<- masterset_graph + scale_x_log10(labels=comma)
    }
  }else{
    if( g_xunit=='$' ){
      masterset_graph <<- masterset_graph + scale_x_continuous(labels=scales::dollar)
    }else if( g_xunit=='%' ){
      masterset_graph <<- masterset_graph + scale_x_continuous(labels=scales::percent)
    }else if( nchar(g_xunit)<=4 ){
      masterset_graph <<- masterset_graph + scale_x_continuous(labels=comma_format( suffix=paste(' ',g_xunit) ))
    }else{
      masterset_graph <<- masterset_graph + scale_x_continuous(labels=comma)
    }
  }
  
  
  
  #--------[ Set Y-Axis Scale ]--------#
  if( is.null(g_ylab) ){ g_ylab = get_meta(g_yvar)$name }
  if( is.null(g_yunit) ){ g_yunit = get_meta(g_yvar)$units }
  if( g_yunit!='$' && g_yunit!='%'){ g_ylab = paste(g_ylab,' (',g_yunit,')',sep='') }
  if( g_ylog ){ g_ylab = paste(g_ylab, "    |    Scale: log(10)", sep='')}
  
  if( g_ylog ){
    if( g_yunit=='$' ){
      masterset_graph <<- masterset_graph + scale_y_log10(labels=scales::dollar)
    }else if( g_yunit=='%' ){
      masterset_graph <<- masterset_graph + scale_y_log10(labels=scales::percent)
    }else if( nchar(g_yunit)<=4 ){
      masterset_graph <<- masterset_graph + scale_y_log10(labels=comma_format( suffix=paste(' ',g_yunit) ))
    }else{
      masterset_graph <<- masterset_graph + scale_y_log10(labels=comma)
    }
  }else{
    if( g_yunit=='$' ){
      masterset_graph <<- masterset_graph + scale_y_continuous(labels=scales::dollar)
    }else if( g_yunit=='%' ){
      masterset_graph <<- masterset_graph + scale_y_continuous(labels=scales::percent)
    }else if( nchar(g_yunit)<=4 ){
      masterset_graph <<- masterset_graph + scale_y_continuous(labels=comma_format( suffix=paste(' ',g_yunit) ))
    }else{
      masterset_graph <<- masterset_graph + scale_y_continuous(labels=comma)
    }
  }
  
  
  
  #--------[ Graph Titles ]--------#
  masterset_graph <<- masterset_graph + xlab(g_xlab)
  masterset_graph <<- masterset_graph + ylab(g_ylab)
  
  if( !is.null(g_size) ){
    g_sizelab = paste('● Size: ',get_meta(g_size)$name,' (',get_meta(g_size)$units,")", sep='')
    masterset_graph <<- masterset_graph + annotate(
      geom  =  'text',
      label =  g_sizelab,
      x     =  Inf,
      y     = -Inf,
      hjust =  1.1,
      vjust = -1.5
    )
  }
  
  if( is.numeric(g_col) ){
    masterset_graph <<- masterset_graph + scale_fill_continuous(
      name = g_llab,
      low  = "blue",
      high = "red"
    )
  }else{
    if( length(unique(g_col)) < 10 ){
      masterset_graph <<- masterset_graph + scale_fill_discrete( name=g_llab )
    }else{
      masterset_graph <<- masterset_graph + guides(fill='none')
    }
  }
  
  
  
  #--------[ Draw Graph ]--------#
  print( masterset_graph )
  
  
  
}